@@ -1,5 +1,5 @@
 /* jpnevulator - serial reader/writer
- * Copyright (C) 2006 Freddy Spierenburg
+ * Copyright (C) 2006-2009 Freddy Spierenburg
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -106,7 +106,7 @@ enum jpnevulatorRtrn jpnevulatorWrite(void) {
 				/* Add a checksum to the message if requested. */
 				if(_jpnevulatorOptions.checksum!=checksumTypeNone) {
 					messageChecksumAdd(message,&index);
-					if(_jpnevulatorOptions.checksumFuckup) {
+					if(boolIsSet(_jpnevulatorOptions.checksumFuckup)) {
 						/* Subtract one from the last checksum byte of the message if the user
 						 * request to fuck up the checksum. */
 						message[index-1]-=1;
@@ -114,7 +114,7 @@ enum jpnevulatorRtrn jpnevulatorWrite(void) {
 				}
 
 				/* Send the message on the line. */
-				if(_jpnevulatorOptions.send) {
+				if(boolIsSet(_jpnevulatorOptions.send)) {
 					if((tty=(struct tty *)listFirst(&_jpnevulatorOptions.tty))!=NULL) {
 						do {
 							/* Delay between bytes if requested. */
@@ -138,7 +138,7 @@ enum jpnevulatorRtrn jpnevulatorWrite(void) {
 				}
 
 				/* Print the message if requested. */
-				if(_jpnevulatorOptions.print) {
+				if(boolIsSet(_jpnevulatorOptions.print)) {
 					for(n=0;n<index;n++) {
 						printf("%02X%c",message[n],n!=(index-1)?' ':'\n');
 					}
@@ -205,8 +205,8 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 	struct timeval timeCurrent,timeLast,*timeoutPtr,timeout;
 	struct tm *time;
 	fd_set readfdsReal,readfdsCopy;
-	struct tty *tty;
-	char ttyNameCopy[sizeof(tty->name)];
+	struct tty *ttyReader,*ttyWriter;
+	char ttyNameCopy[sizeof(ttyReader->name)];
 	int nfds;
 
 	/* Open our output file. */
@@ -226,7 +226,7 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 	}
 
 	/* Allocate memory for the ascii data to print if desired. */
-	if(_jpnevulatorOptions.ascii) {
+	if(boolIsSet(_jpnevulatorOptions.ascii)) {
 		asciiSize=(sizeof(ascii[0])*_jpnevulatorOptions.width)+1;
 		ascii=(char *)malloc(asciiSize);
 		if(ascii==NULL) {
@@ -249,11 +249,11 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 	 * so we don't have to parse our list of tty devices every time we iterate. */
 	FD_ZERO(&readfdsCopy);
 	nfds=0;
-	if((tty=(struct tty *)listFirst(&_jpnevulatorOptions.tty))!=NULL) {
+	if((ttyReader=(struct tty *)listFirst(&_jpnevulatorOptions.tty))!=NULL) {
 		do {
-			FD_SET(tty->fd,&readfdsCopy);
-			nfds=max(nfds,tty->fd);
-		} while((tty=(struct tty *)listNext(&_jpnevulatorOptions.tty))!=NULL);
+			FD_SET(ttyReader->fd,&readfdsCopy);
+			nfds=max(nfds,ttyReader->fd);
+		} while((ttyReader=(struct tty *)listNext(&_jpnevulatorOptions.tty))!=NULL);
 	} else {
 		fprintf(stderr,"%s: No available tty to read from\n",PROGRAM_NAME);
 		jpnevulatorGarbageCollect();
@@ -269,7 +269,7 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 	 * values for the received bytes. In that case we use the timeout to display
 	 * the ASCII values automatically, otherwise they will never appear when less
 	 * then the line length bytes are received and no more bytes are coming. */
-	if(_jpnevulatorOptions.ascii) {
+	if(boolIsSet(_jpnevulatorOptions.ascii)) {
 		timeoutPtr=&timeout;
 	} else {
 		timeoutPtr=NULL;
@@ -289,21 +289,21 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 		rtrn=select(nfds+1,&readfdsReal,NULL,NULL,timeoutPtr);
 		if(rtrn==-1) {
 		} else if(rtrn) {
-			if((tty=(struct tty *)listFirst(&_jpnevulatorOptions.tty))!=NULL) {
+			if((ttyReader=(struct tty *)listFirst(&_jpnevulatorOptions.tty))!=NULL) {
 				do {
-					if(FD_ISSET(tty->fd,&readfdsReal)) {
-						bytesRead=read(tty->fd,message,_jpnevulatorOptions.size);
+					if(FD_ISSET(ttyReader->fd,&readfdsReal)) {
+						bytesRead=read(ttyReader->fd,message,_jpnevulatorOptions.size);
 						if(bytesRead>0) {
 							timeLast=timeCurrent;
 							gettimeofday(&timeCurrent,NULL);
 							if(
-								_jpnevulatorOptions.timingPrint&&
-								((memcmp(ttyNameCopy,tty->name,sizeof(ttyNameCopy))!=0)||
+								boolIsSet(_jpnevulatorOptions.timingPrint)&&
+								((memcmp(ttyNameCopy,ttyReader->name,sizeof(ttyNameCopy))!=0)||
 								(((((timeCurrent.tv_sec-timeLast.tv_sec)*1000000L)+timeCurrent.tv_usec)-timeLast.tv_usec)>_jpnevulatorOptions.timingDelta))
 							) {
 								time=localtime(&timeCurrent.tv_sec);
 								if(bytesWritten!=0) {
-									if(_jpnevulatorOptions.ascii) {
+									if(boolIsSet(_jpnevulatorOptions.ascii)) {
 										for(index=bytesWritten;index<_jpnevulatorOptions.width;index++) {
 											fprintf(output,"   ");
 										}
@@ -322,18 +322,18 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 								 * be displayed as part of the printing of the timing. It's
 								 * way to confusing otherwise. */
 								if(listElements(&_jpnevulatorOptions.tty)>1) {
-									fprintf(output," %s",ttyPrint(tty));
+									fprintf(output," %s",ttyPrint(ttyReader));
 								}
-								memcpy(ttyNameCopy,tty->name,sizeof(ttyNameCopy));
+								memcpy(ttyNameCopy,ttyReader->name,sizeof(ttyNameCopy));
 								fprintf(output,"\n");
 								bytesWritten=0;
 							} else {
 								if(
 									(listElements(&_jpnevulatorOptions.tty)>1)&&
-									(memcmp(ttyNameCopy,tty->name,sizeof(ttyNameCopy))!=0)
+									(memcmp(ttyNameCopy,ttyReader->name,sizeof(ttyNameCopy))!=0)
 								) {
 									if(bytesWritten!=0) {
-										if(_jpnevulatorOptions.ascii) {
+										if(boolIsSet(_jpnevulatorOptions.ascii)) {
 											for(index=bytesWritten;index<_jpnevulatorOptions.width;index++) {
 												fprintf(output,"   ");
 											}
@@ -342,14 +342,14 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 										}
 										fprintf(output,"\n");
 									}
-									fprintf(output,"%s\n",ttyPrint(tty));
-									memcpy(ttyNameCopy,tty->name,sizeof(ttyNameCopy));
+									fprintf(output,"%s\n",ttyPrint(ttyReader));
+									memcpy(ttyNameCopy,ttyReader->name,sizeof(ttyNameCopy));
 									bytesWritten=0;
 								}
 							}
 							for(index=0;index<bytesRead;index++) {
 								if(bytesWritten>=_jpnevulatorOptions.width) {
-									if(_jpnevulatorOptions.ascii) {
+									if(boolIsSet(_jpnevulatorOptions.ascii)) {
 										fprintf(output,"\t%s",ascii);
 										memset(ascii,'\0',asciiSize);
 									}
@@ -359,26 +359,48 @@ enum jpnevulatorRtrn jpnevulatorRead(void) {
 									if(bytesWritten!=0) fprintf(output," ");
 								}
 								if((bytesWritten==0)&&boolIsSet(_jpnevulatorOptions.byteCountDisplay)) {
-									fprintf(output,"%08lX\t",tty->byteCount);
+									fprintf(output,"%08lX\t",ttyReader->byteCount);
 								}
 								fprintf(output,"%02X",message[index]);
 								/* Increase the byte count for this tty. */
-								tty->byteCount++;
-								if(_jpnevulatorOptions.ascii) {
+								ttyReader->byteCount++;
+								if(boolIsSet(_jpnevulatorOptions.ascii)) {
 									ascii[bytesWritten]=isprint(message[index])?message[index]:'.';
 								}
 								bytesWritten++;
 							}
+							/* Does the user want to pass the data between all the tty's? */
+							if(boolIsSet(_jpnevulatorOptions.pass)) {
+								struct listElement *ttyListPosition;
+								/* Save the current position in the tty list, since we are about to traverse
+									 it again. */
+								ttyListPosition=listCurrentPositionSave(&_jpnevulatorOptions.tty);
+								/* Traverse the tty list again in search for all the other (not this read) tty's. For
+								 * every tty found write the read message to it. */
+								if((ttyWriter=(struct tty *)listFirst(&_jpnevulatorOptions.tty))!=NULL) {
+									do {
+										if(ttyWriter->fd!=ttyReader->fd) {
+											ssize_t n;
+											n=write(ttyWriter->fd,message,bytesRead);
+											if(n<0) {
+												fprintf(stderr,"%s: %s: write of %ld bytes failed(%ld).\n",PROGRAM_NAME,ttyPrint(ttyWriter),bytesRead,n);
+											}
+										}
+									} while((ttyWriter=(struct tty *)listNext(&_jpnevulatorOptions.tty))!=NULL);
+								}
+								/* Restore the current position for the tty list again. */
+								listCurrentPositionLoad(&_jpnevulatorOptions.tty,ttyListPosition);
+							}
 							fflush(output);
 						}
 					}
-				} while((tty=(struct tty *)listNext(&_jpnevulatorOptions.tty))!=NULL);
+				} while((ttyReader=(struct tty *)listNext(&_jpnevulatorOptions.tty))!=NULL);
 			}
 		} else {
 			/* No more bytes received within the timeout, so let's write our
 			 * ASCII data if necessary. */
 			if(bytesWritten!=0) {
-				if(_jpnevulatorOptions.ascii) {
+				if(boolIsSet(_jpnevulatorOptions.ascii)) {
 					for(index=bytesWritten;index<_jpnevulatorOptions.width;index++) {
 						fprintf(output,"   ");
 					}
