@@ -1,5 +1,5 @@
 /* jpnevulator - serial reader/writer
- * Copyright (C) 2006 Freddy Spierenburg
+ * Copyright (C) 2006-2009 Freddy Spierenburg
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,6 +44,10 @@ extern enum listRtrn listAppend(list_t *,void *);
 extern enum listRtrn listInsert(list_t *,void *);
 extern void listRemove(list_t *,void (*)(void *));
 extern void *listCurrent(list_t *);
+/* These next two are routines to temporary freeze the current pointer in the list. This
+ * way one can traverse the list within a traverse of the list itself. */
+#define listCurrentPositionSave(x) ((x)->current)
+#define listCurrentPositionLoad(x,y) ((x)->current=y)
 extern void *listFirst(list_t *);
 extern void *listNext(list_t *);
 extern void *listPrevious(list_t *);
