@@ -1,5 +1,5 @@
 /* jpnevulator - serial reader/writer
- * Copyright (C) 2006 Freddy Spierenburg
+ * Copyright (C) 2006-2009 Freddy Spierenburg
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -39,21 +39,22 @@ struct jpnevulatorOptions {
 	enum checksumType checksum;
 	unsigned int crc16Poly;
 	unsigned char crc8Poly;
-	char checksumFuckup;
+	bool_t checksumFuckup;
 	char io[256];
 	list_t tty;
 	int size;
-	char send;
-	char print;
+	bool_t send;
+	bool_t print;
 	unsigned long delayLine;
 	unsigned long delayByte;
 	enum actionType action;
 	int width;
-	char timingPrint;
+	bool_t timingPrint;
 	unsigned long timingDelta;
-	char ascii;
+	bool_t ascii;
 	char *aliasSeparator;
 	bool_t byteCountDisplay;
+	bool_t pass;
 };
 
 enum optionsRtrn {
