@@ -1,5 +1,5 @@
 /* jpnevulator - serial reader/writer
- * Copyright (C) 2006 Freddy Spierenburg
+ * Copyright (C) 2006-2009 Freddy Spierenburg
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -34,7 +34,7 @@ enum ttyRtrn {
 	ttyRtrnList
 };
 
-#define ttyPrint(x) (strlen((tty)->alias)>0?(tty)->alias:(tty)->name)
+#define ttyPrint(x) (strlen((x)->alias)>0?(x)->alias:(x)->name)
 extern void ttyInitialize(void);
 extern enum ttyRtrn ttyAdd(char *);
 extern void ttyDestroy(void);
